@@ -10,11 +10,11 @@ from rasa.constants import (
     DEFAULT_ENDPOINTS_PATH,
     DEFAULT_MODELS_PATH,
     DEFAULT_RESULTS_PATH,
-    DEFAULT_NLU_RESULTS_PATH,
     CONFIG_SCHEMA_FILE,
 )
 import rasa.utils.validation as validation_utils
 import rasa.cli.utils as cli_utils
+import rasa.utils.io as io_utils
 
 logger = logging.getLogger(__name__)
 
@@ -67,8 +67,7 @@ def test_core(args: argparse.Namespace) -> None:
     stories = data.get_core_directory(stories)
     output = args.out or DEFAULT_RESULTS_PATH
 
-    if not os.path.exists(output):
-        os.makedirs(output)
+    io_utils.create_directory(output)
 
     if isinstance(args.model, list) and len(args.model) == 1:
         args.model = args.model[0]
@@ -95,21 +94,19 @@ def test_core(args: argparse.Namespace) -> None:
 
 def test_nlu(args: argparse.Namespace) -> None:
     from rasa import data
-    import rasa.utils.io as io_utils
     from rasa.test import compare_nlu_models, perform_nlu_cross_validation, test_nlu
 
     nlu_data = cli_utils.get_validated_path(args.nlu, "nlu", DEFAULT_DATA_PATH)
     nlu_data = data.get_nlu_directory(nlu_data)
+    output = args.out or DEFAULT_RESULTS_PATH
+    args.errors = not args.no_errors
+
+    io_utils.create_directory(output)
 
     if args.config is not None and len(args.config) == 1:
         args.config = os.path.abspath(args.config[0])
         if os.path.isdir(args.config):
-            config_dir = args.config
-            config_files = os.listdir(config_dir)
-            args.config = [
-                os.path.join(config_dir, os.path.abspath(config))
-                for config in config_files
-            ]
+            args.config = io_utils.list_files(args.config)
 
     if isinstance(args.config, list):
         logger.info(
@@ -127,11 +124,10 @@ def test_nlu(args: argparse.Namespace) -> None:
                 config_files.append(file)
             except validation_utils.InvalidYamlFileError:
                 logger.debug(
-                    "Ignoring file '{}' as it is not a valid config file.".format(file)
+                    f"Ignoring file '{file}' as it is not a valid config file."
                 )
                 continue
 
-        output = args.report or DEFAULT_NLU_RESULTS_PATH
         compare_nlu_models(
             configs=config_files,
             nlu=nlu_data,
@@ -144,12 +140,13 @@ def test_nlu(args: argparse.Namespace) -> None:
         config = cli_utils.get_validated_path(
             args.config, "config", DEFAULT_CONFIG_PATH
         )
-        perform_nlu_cross_validation(config, nlu_data, vars(args))
+        perform_nlu_cross_validation(config, nlu_data, output, vars(args))
     else:
         model_path = cli_utils.get_validated_path(
             args.model, "model", DEFAULT_MODELS_PATH
         )
-        test_nlu(model_path, nlu_data, vars(args))
+
+        test_nlu(model_path, nlu_data, output, vars(args))
 
 
 def test(args: argparse.Namespace):
