@@ -16,8 +16,8 @@ import re
 import string
 from text2digits import text2digits
 from metrics.constants import APP_NAV
-
-
+from webservice_connector.voltage import Voltage
+voltage_client = Voltage()
 t2d = text2digits.Text2Digits()
 
 from metrics.constants import ACTION_PREDICT_EXECUTE_TIME
@@ -672,14 +672,29 @@ class MessageProcessor:
                 or tracker.latest_message.intent.get("name") == USER_INTENT_RESTART
         )
 
-    async def send_app_nav(self, conversation_id, app_nav):
-        method = 'POST'
+   
+    async def send_app_nav_async(self, conversation_id, app_nav):
         payload = {
             'appNavs': [app_nav]
         }
-        endpoint = f"{FC_ENDPOINT}/Accounts/{FC_ACCOUNT_ID}/Calls/{conversation_id}/AppNavs"
-        requests.post(endpoint, json=payload, auth=(FC_ACCOUNT_ID,FC_ACCOUNT_ID))
-        return None
+        async with aiohttp.ClientSession() as session:
+            fc_account_res = await voltage_client.fpeDecrypt(FC_ACCOUNT_ID, 'ASCII_ALPHANUM_PIN')
+            fc_token_res = await voltage_client.fpeDecrypt(FC_ACCOUNT_TOKEN, 'ASCII_ALPHANUM_PIN')
+            fc_account = fc_account_res.get('text',None)
+            fc_token = fc_token_res.get('text',None)
+            
+            fc_auth = aiohttp.BasicAuth(fc_account, fc_token)
+            endpoint = f"{FC_ENDPOINT}/Accounts/{fc_account}/Calls/{fc_token}/AppNavs"
+
+            try:
+                async with session.post(endpoint, auth=fc_auth, data=payload, ssl=False) as response:
+                    logger.info('[{}] - cdr submission: {}'.format(conversation_id, response.status))
+            except aiohttp.ClientConnectorError as e:
+                logger.debug('[{}] - cdr connection error: {}'.format(conversation_id, str(e)))
+            except Exception as e:
+                logger.debug('[{}] - cdr submission failed: {}'.format(conversation_id, str(e)))
+    
+    
 
 
     def log_appNav(self, sender_id: str, message: UserUttered):
@@ -690,7 +705,7 @@ class MessageProcessor:
                     'value:': 'disconnect'
                 }
                 logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
-                asyncio.create_task(self.send_app_nav(sender_id, app_nav))
+                #asyncio.create_task(self.send_app_nav_async(sender_id, app_nav))
                 APP_NAV.labels(message._metadata['template_name'], 'disconnect').inc()
 
             elif message._metadata and message._metadata.get('message_type', None) == 'SPOKEN':
@@ -700,7 +715,7 @@ class MessageProcessor:
                     'value:': 'NO_NTENT'
                     }
                     logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
-                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    #asyncio.create_task(self.send_app_nav_async(sender_id,app_nav))
                     APP_NAV.labels(message._metadata['template_name'], message._metadata['reporting_error_type']).inc()
                     
                 elif message.missed_intent and message.intent['confidence'] < message._metadata['nlu_threshold']:
@@ -709,7 +724,7 @@ class MessageProcessor:
                     'value:': 'NO_NTENT'
                     }
                     logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
-                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    #asyncio.create_task(self.send_app_nav_async(sender_id,app_nav))
                     APP_NAV.labels(message._metadata['template_name'], 'no_match').inc()
                     
                 elif message.intent['confidence'] > message._metadata['nlu_threshold']:
@@ -718,7 +733,7 @@ class MessageProcessor:
                     'value:': message.intent
                     }
                     logger.info('[{}] - APP NAV: {}'.format(sender_id, app_nav))
-                    asyncio.create_task(self.send_app_nav(sender_id,app_nav))
+                    #asyncio.create_task(self.send_app_nav_async(sender_id,app_nav))
                     APP_NAV.labels(message._metadata['template_name'], 'match').inc()
                 else:
                     logger.debug("[{}] - APP NAV: Error logging app_nav: {}".format(sender_id, message))
