@@ -35,6 +35,7 @@ from rasa.core.utils import replace_floats_with_decimals
 from rasa.utils.common import class_from_module_path
 from rasa.utils.timer import timeit
 from rasa.utils.endpoints import EndpointConfig
+from rasa.utils.redactor import Redactor
 
 if typing.TYPE_CHECKING:
     from sqlalchemy.engine.url import URL
@@ -54,6 +55,7 @@ class TrackerStore:
         self.domain = domain
         self.event_broker = event_broker
         self.max_event_history = None
+        self.redactor = Redactor()
 
     @staticmethod
     def find_tracker_store(
@@ -221,11 +223,28 @@ class TrackerStore:
         start = time.perf_counter()
         form_error = False
         prediction_error = False
+        step_number = offset
 
         for evt in list(itertools.islice(evts, offset, len(evts))):
             body = {"sender_id": tracker.sender_id}
             evt_dict = evt.as_dict()
+            metadata = evt_dict.get('metadata', {})
+            event_type = evt_dict.get('event')
+            evt_text = evt_dict.get('text', None)
+            not_disconnect = evt_text != '/disconnect' and evt_text != '/restart'
+            
+            if len(metadata) > 0:
+                should_redact = metadata.get('should_redact', False)
+            else:
+                should_redact = False
+            if str(event_type).lower() == 'user' and not_disconnect:
+                if should_redact:
+                    evt_dict = self.redactor.redact_event_complete(evt_dict)
+                else:
+                    evt_dict = self.redactor.redact_event(evt_dict)
             body.update(evt_dict)
+            body["step_number"] = step_number
+            step_number += 1
             if TrackerStore.is_form_error(tracker, evt_dict):
                 form_error = True
             elif TrackerStore.is_prediction_error(evt_dict):
@@ -237,7 +256,6 @@ class TrackerStore:
                 elif prediction_error:
                     body["metadata"]["prediction_error"] = True
                     prediction_error = False
-
             tracker.add_conversation_steps(evt)
             self.event_broker.publish(body)
 
@@ -528,7 +546,7 @@ class RedisClusterTrackerStore(TrackerStore):
         self.max_connections = max_connections
         self.delay = 5
         self.max_retries = 3
-        self.socket_timeout = int(os.environ.get('REDIS_SOCKET_TIMEOUT', 2))
+        self.socket_timeout = float(os.environ.get('REDIS_SOCKET_TIMEOUT', 2.0))
         self.skip_full_coverage_check = bool(os.environ.get('SKIP_FULL_COVERAGE_CHECK', True))
         self.health_check_retries = 0
         self.red_cluster = RedisCluster(
@@ -536,7 +554,8 @@ class RedisClusterTrackerStore(TrackerStore):
             max_connections=int(max_connections),
             retry_on_timeout=False,
             socket_connect_timeout=self.socket_timeout,
-            socket_timeout=self.socket_timeout
+            socket_timeout=self.socket_timeout,
+            reinitialize_steps=1
         )
         self.record_exp = record_exp
         super(RedisClusterTrackerStore, self).__init__(domain, event_broker)
@@ -567,7 +586,8 @@ class RedisClusterTrackerStore(TrackerStore):
                 max_connections=int(self.max_connections),
                 retry_on_timeout=False,
                 socket_connect_timeout=self.socket_timeout,
-                socket_timeout=self.socket_timeout
+                socket_timeout=self.socket_timeout,
+                reinitialize_steps=1
             )
             return True
         except Exception as exp:
@@ -677,7 +697,7 @@ class RedisClusterTrackerStore(TrackerStore):
                     f'[REDIS_CLUSTER_ERROR] - Failed to ping Redis, error count: {self.health_check_retries}. Error: {exp}')
                 self.recreate_connection()
 
-            time.sleep(5)
+            time.sleep(10)
             if self.health_check_retries == self.max_retries:
                 self.health_status = False
                 logger.error(
