@@ -2,6 +2,7 @@ import logging
 import tempfile
 from typing import Tuple, Text
 from unittest.mock import Mock
+from json import loads
 
 import pytest
 from _pytest.logging import LogCaptureFixture
@@ -10,7 +11,7 @@ from moto import mock_dynamodb2
 
 from rasa.core.channels.channel import UserMessage
 from rasa.core.domain import Domain
-from rasa.core.events import SlotSet, ActionExecuted, Restarted
+from rasa.core.events import SlotSet, ActionExecuted, Restarted, UserUttered, BotUttered
 from rasa.core.tracker_store import (
     TrackerStore,
     InMemoryTrackerStore,
@@ -24,6 +25,8 @@ from rasa.core.trackers import DialogueStateTracker
 from rasa.utils.endpoints import EndpointConfig, read_endpoint_config
 from tests.conftest import assert_log_emitted
 from tests.core.conftest import DEFAULT_ENDPOINTS_FILE
+import rasa.core.brokers.utils as broker_utils
+from tests.core.utilities import get_tracker
 
 domain = Domain.load("data/test_domains/default.yml")
 
@@ -379,3 +382,25 @@ def test_set_fail_safe_tracker_store_domain(default_domain: Domain):
     assert failsafe_store.domain is default_domain
     assert tracker_store.domain is failsafe_store.domain
     assert fallback_tracker_store.domain is failsafe_store.domain
+
+def test_stream_events_step_assignment():
+    actual_steps_sequence = []
+    expected_steps_sequence = [0, 1, 2, 3, 4, 5, 6, 7, 8]
+    broker = broker_utils.from_endpoint_config(read_endpoint_config("data/test_endpoints/event_brokers/pika_endpoint.yml", "event_broker"))
+    tracker_store = InMemoryTrackerStore(domain, broker)
+    events = [
+        UserUttered("/greet", {"name": "greet", "confidence": 1.0}, []),
+        BotUttered("one", data={"custom": {"type": "capture_utterance"}}),
+        BotUttered("two", data={"custom": {"type": "dtmf"}}),
+        UserUttered("sip_info", {"name": "goodbye", "confidence": 1.0}, []),
+        BotUttered("three", data={"custom": {"type": "get_speech"}}),
+        BotUttered("four", data={"custom": {"type": "send_sip_info"}}),
+        UserUttered("/goodbye", {"name": "goodbye", "confidence": 1.0}, []),
+        BotUttered("five", data={"custom": {"type": "not_app_event"}}),
+        BotUttered("six", data={"custom": {"type": "hangup"}})
+    ]
+    tracker = get_tracker(events)
+    tracker_store.stream_events(tracker)
+    while broker._unpublished_messages:
+        actual_steps_sequence.insert(0, loads(broker._unpublished_messages.pop())["step_number"])
+    assert actual_steps_sequence == expected_steps_sequence
\ No newline at end of file
