@@ -2137,10 +2137,19 @@ event_with_cc_num_2_redacted = {
 def test_string_redaction():
     assert redactor.redact_string(msg_with_pci) == msg_without_pci
 
+def test_string_redaction_spacing():
+    msg = "1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1"
+    msg_redacted = '****************'
+    assert redactor.redact_string(msg) == msg_redacted
+
 
 def test_string_redaction_strict():
     assert redactor.redact_string_strict(msg_with_pci) == msg_without_pci_strict
 
+def test_string_redaction_spacing_strict():
+    msg = "1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1"
+    msg_redacted = '****************'
+    assert redactor.redact_string_strict(msg) == msg_redacted
 
 def test_redact_int_len_1():
     assert redactor.redact_int(1) == 1
